<?php

namespace SupplyTek\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use SupplyTek\Utilities\SessionHandler;
use Zeedhi\Framework\Report\ReportService;
use SupplyTek\Utilities\CrudSupplyTek;

/**
 * Class prepares parameters to generate BIRT reports by returning them to a URL
 * 
 * @author Grazielle Ferreira Antunes
 */

class GraphicsReportsController extends \Zeedhi\Framework\Controller\Report {
    
    protected $reportService;
    protected $birtStrategy;
    protected $sessionHandler;
    protected $crud;
    
    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param object $reportService - Class of zeedhi required for the operation of BIRT reports.
     * @param object $birtStrategy - Class of zeedhi required for the operation of BIRT reports.
     * @param object $sessionHandler - Class to handle data from PHP Session.
     * @param CrudSupplyTek $crudSupplyTek Object that allows to perform queries from datasource
     * 
     * @return void
     * 
     * @access public
     * 
     * @author Grazielle Ferreira Antunes
     */
    public function __construct($reportService, $birtStrategy, $sessionHandler, $crudSupplyTek) {
        $this->reportService     = $reportService;
        $this->birtStrategy      = $birtStrategy;
        $this->sessionHandler    = $sessionHandler;
        $this->crud = $crudSupplyTek;
	}
	
	/**
     * Maps available reports and their respective parameters that 
     * will be received by BIRT to generate the report
     * 
     * @author Grazielle Ferreira Antunes
     */ 
	protected $reportMapping = array (
            'ConsumoPorCategoria' =>
                array (
                    'strategy' => 'BIRT',
                    'parameterMapping' =>
                        array (
                            "BRANCH"       => "BRANCH",
                            "CATEGORIES"   => "CATEGORY_ID",
                            "OPERATOR_ID"  => "OPERATOR_ID",
                            "FIRST_MONTH"  => "FIRST_MONTH",
                            "LAST_MONTH"   => "LAST_MONTH",
                            "NMBRANCH"     => "NMBRANCH",
                            "NMCATEGORIES" => "NMCATEGORIES",
                            "NRORG"        => "NRORG",
                            "NMOPERATOR"   => "NMOPERATOR"
                        )
                ),
            'OrcamentoUnidade' =>
                array (
                    'strategy' => 'BIRT',
                    'parameterMapping' =>
                        array (
                            "BRANCH"      => "BRANCH",
                            "PERIOD"      => "PERIOD",
                            "OPERATOR_ID" => "OPERATOR_ID",
                            "NRORG"       => "NRORG",
                            "NMBRANCH"    => "NMBRANCH",
                            "NMOPERATOR"  => "NMOPERATOR"
                        )
                ),
            'PedidosUnidade' =>
                array (
                    'strategy' => 'BIRT',
                    'parameterMapping' =>
                        array (
                            "OPERATOR_ID" => "OPERATOR_ID",
                            "BRANCH"      => "BRANCH",
                            "FIRST_DAY"   => "FIRST_DAY",
                            "LAST_DAY"    => "LAST_DAY",
                            "NRORG"       => "NRORG",
                            "NMBRANCH"    => "NMBRANCH",
                            "NMOPERATOR"  => "NMOPERATOR"
                        )
                ),
            'OrcamentoPorCategoria' =>
                array (
                    'strategy' => 'BIRT',
                    'parameterMapping' =>
                        array (
                            "BRANCH"      => "BRANCH",
                            "PERIOD"      => "PERIOD",
                            "OPERATOR_ID" => "OPERATOR_ID",
                            "PERIOD1"      => "FIRST_MONTH",
                            "PERIOD2"      => "LAST_MONTH",
                            "NMBRANCH"    => "NMBRANCH",
                            "NMOPERATOR" => "NMOPERATOR",
                            "NRORG" => "NRORG"
                        )
                ),
            
    );
    
    /**
     * Prepare the parameters for mapping the report including and/or 
     * changing the required parameters in the request row
     * and set the report format in birtStrategy.
     * 
     * @param Request $request – Object that receives data from the caller.
     * @param Response $response –  Object that sends back data to the caller.
     * 
     * @access public
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    public function beforeProcessReport(Request\Row $request, Response $response) {
        
        $row = $request->getRow();
        
        $row["OPERATOR_ID"] = $this->sessionHandler->getSessionData('CDOPERADOR');
        $row['NRORG'] = $this->sessionHandler->getSessionData('NRORG');
        
        $row['__report_name'] = $this->defineNameReport($row['REPORT']);
        
        $param = array("OPERATOR_ID" => $row['OPERATOR_ID']);
        $row['NMOPERATOR'] = $this->crud->executeQuery('operator', $param)[0]['NMOPERADOR'];

        if($row['__report_name'] == 'ConsumoPorCategoria'){
            if(count($row['CATEGORIES'])) {
                
                $row['CATEGORIES'] = $this->concatCategories($row['CATEGORIES']);
                
                $param = array("CATEGORY_ID" => $row['CATEGORIES']);
                $nameCategories = $this->crud->executeQuery('categories', $param);
                $row['NMCATEGORIES'] = $this->concatNameCategories($nameCategories);
                
            } else {
                
                $row['CATEGORIES'] = 'T';
                $row['NMCATEGORIES'] = 'T';
            }
            
            list($row['FIRST_MONTH'], $row['LAST_MONTH']) = $this->getPeriod($row['MONTH']);
            
        } else if($row['__report_name'] == 'PedidosUnidade') {
            
            list($row['FIRST_DAY'], $row['LAST_DAY']) = $this->getPeriodMonth($row['MONTH']);
            
        } else {
            
            list($row['PERIOD'], $isNull) = $this->getPeriodMonth($row['MONTH']);
            $row['PERIOD1'] = $row['PERIOD'];
            $row['PERIOD2'] = $row['PERIOD'];
        }
        
        
        $row['BRANCH'] = isset($row['BRANCH']) ? $row['BRANCH'] : $this->sessionHandler->getSessionData('CDFILIAL');
        if($row['BRANCH'] != 'T') {
            $param = array("BRANCH" => $row['BRANCH']);
            $row['NMBRANCH'] = $this->crud->executeQuery('branch', $param)[0]['NMFILIAL'];
        }else{
            $row['NMBRANCH'] = 'T';
        }
        
        $this->birtStrategy->setReportFormat($row['FORMAT']);
    }
    
    /**
     * Define the report name according to the file name in the reports folder.
     * 
     * @param String $name – Report name received on request.
     * 
     * @return String Reports name according to the files in the reports folder.
     * 
     * @access private
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    private function defineNameReport($name) {
        switch ($name) {
            case "EXPENSES_BY_CATEGORY":
                return "ConsumoPorCategoria";
                break;
            case "REQUESTER_BUDGET":
                return "OrcamentoUnidade";
                break;
            case "REQUESTER_ORDERS":
                return "PedidosUnidade";
                break;
            case "BUDGET_BY_CATEGORY":
                return "OrcamentoPorCategoria";
                break;
        }
    }
    
    /**
     * Concatenates the categories to be received by the report query.
     * 
     * @param array $categories – Array with category codes received on request
     * 
     * @return String Category codes concatenated by underline
     * 
     * @access private
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    private function concatCategories($categories) {
        return  '_' . implode('_', $categories) . '_';
    }
    
    /**
     * Concatenates the name of the categories to be received report header.
     * 
     * @param array $nameCategories – Array with categories name received on service
     * 
     * @return String Category names concatenated by underline
     * 
     * @access private
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    private function concatNameCategories($nameCategories) {
        $arrayNmCategories = array();
        foreach ($nameCategories as $nameCategory) {
            array_push($arrayNmCategories, $nameCategory['NAME']);
        }
        
        return implode(', ', $arrayNmCategories);
    }
    
    /**
     * Returns the period between the first day of the month received 
     * by the request and the first day of the month corresponding to 12 months ago.
     * 
     * @param array $date – Month and year received by the request
     * 
     * @return String $firstMonth First day of the month corresponding to 12 months prior to the month received in the request
     * @return String $lastMonth First day of the month received by the request 
     * 
     * @access private
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    private function getPeriod ($date)
    {
        
        $date = \DateTime::createFromFormat('m/y', $date);
        
        if (!$date) {
            throw new \Exception("Wrong date format.");
        }

        $date->modify('first day of this month');
        $lastMonth = $date->format('d/m/Y');
        
        $date->modify('-11 month');
        $firstMonth = $date->format('d/m/Y');
        
        return array($firstMonth, $lastMonth);
    }
    
    /**
     * Returns the period between the first day and the last day of the month received in the request..
     * 
     * * @param array $date – Month and year received by the request
     * 
     * @return String $firstDay First day of the month received on request
     * @return String $lastDay Last day of the month received by the request 
     * 
     * @access private
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    private function getPeriodMonth ($date)
    {
        $date = \DateTime::createFromFormat('m/y', $date);
        
        if (!$date) {
            throw new \Exception("Wrong date format.");
        }

        $date->modify('first day of this month');
        $firstDay = $date->format('d/m/Y');
        
        $date->modify('last day of this month');
        $lastDay = $date->format('d/m/Y');
        
        return [$firstDay, $lastDay];
    }
	
}